# __author__ = "Donghoon Lee"
# __copyright__ = "Copyright 2017, Gerstein Lab"
# __credits__ = ["Donghoon Lee"]
# __license__ = "GPL"
# __version__ = "1.0.0"
# __maintainer__ = "Donghoon Lee"
# __email__ = "donghoon.lee@yale.edu"

###
### R script examples for calculating direct connection rewiring
###

library(data.table)
library(reshape2)
library(ggplot2)

setwd("/path/to/data")

# functions
loadTIPNet <- function(path_tip_nor,path_tip_can,path_enh_nor,path_enh_can){
  
  # load edge list
  
  edge_tip_nor <- fread(path_tip_nor,header=F,sep="\t")
  edge_tip_can <- fread(path_tip_can,header=F,sep="\t")
  edge_enh_nor <- fread(path_enh_nor,header=F,sep="\t")
  edge_enh_can <- fread(path_enh_can,header=F,sep="\t")
  
  edge_merge_nor <- rbind(edge_tip_nor,edge_enh_nor)
  edge_merge_can <- rbind(edge_tip_can,edge_enh_can)
  
  colnames(edge_merge_nor) <- c("SAMPLE","TF","GENE","SRC")
  colnames(edge_merge_can) <- c("SAMPLE","TF","GENE","SRC")
  
  normal <- subset(edge_merge_nor,select=c(TF,GENE,SRC))
  cancer <- subset(edge_merge_can,select=c(TF,GENE,SRC))
  normal$SAMPLE <- "NORMAL"
  cancer$SAMPLE <- "CANCER"

  return(rbind(unique(normal),unique(cancer)))
}
loadTSSNet <- function(path_tss_nor,path_tss_can,path_enh_nor,path_enh_can){
  
  # load edge list
  
  edge_tss_nor <- fread(path_tss_nor,header=F,sep="\t")
  edge_tss_can <- fread(path_tss_can,header=F,sep="\t")
  colnames(edge_tss_nor) <- c("SAMPLE","TF","GENE","SRC")
  colnames(edge_tss_can) <- c("SAMPLE","TF","GENE","SRC")
  edge_tss_nor <- subset(edge_tss_nor,select=c(TF,GENE,SRC))
  edge_tss_can <- subset(edge_tss_can,select=c(TF,GENE,SRC))
  
  edge_enh_nor <- fread(path_enh_nor,header=F,sep="\t")
  edge_enh_can <- fread(path_enh_can,header=F,sep="\t")
  colnames(edge_enh_nor) <- c("SAMPLE","TF","GENE","SRC")
  colnames(edge_enh_can) <- c("SAMPLE","TF","GENE","SRC")
  edge_enh_nor <- subset(edge_enh_nor,select=c(TF,GENE,SRC))
  edge_enh_can <- subset(edge_enh_can,select=c(TF,GENE,SRC))
  
  edge_merge_nor <- rbind(edge_tss_nor,edge_enh_nor)
  edge_merge_can <- rbind(edge_tss_can,edge_enh_can)
  
  normal <- subset(edge_merge_nor,select=c(TF,GENE,SRC))
  cancer <- subset(edge_merge_can,select=c(TF,GENE,SRC))
  normal$SAMPLE <- "NORMAL"
  cancer$SAMPLE <- "CANCER"
  
  return(rbind(unique(normal),unique(cancer)))
}
networkAnalysis <- function(net, commonTF, path_biotype){
  
  # subset commonTF #
  
  TFnet <- subset(net,TF %in% commonTF)
  
  # subset protein_coding GENE #
  
  dgene.type <- fread(path_biotype,header=T,sep="\t")
  dgene.type <- data.frame(dgene.type)
  dgene.type <- data.frame(lapply(dgene.type, as.character), stringsAsFactors=FALSE)
  gene.coding <- dgene.type$gene_name[dgene.type$gene_type=="protein_coding"]
  
  TFnet <- subset(TFnet, GENE %in% gene.coding)
  
  # node #
  
  node_TF_G <- unique(subset(TFnet, SAMPLE=="NORMAL")$TF)
  node_TF_K <- unique(subset(TFnet, SAMPLE=="CANCER")$TF)
  
  CN_TF <- intersect(node_TF_G,node_TF_K)
  GN_TF <- setdiff(node_TF_K,node_TF_G)
  LN_TF <- setdiff(node_TF_G,node_TF_K)
  
  print(paste("CN TF:",length(CN_TF),sep=" "))
  print(paste("GN TF:",length(GN_TF),sep=" "))
  print(paste("LN TF:",length(LN_TF),sep=" "))
  
  node_GENE_G <- unique(subset(TFnet, SAMPLE=="NORMAL")$GENE)
  node_GENE_K <- unique(subset(TFnet, SAMPLE=="CANCER")$GENE)
  
  CN_GENE <- intersect(node_GENE_G,node_GENE_K)
  GN_GENE <- setdiff(node_GENE_K,node_GENE_G)
  LN_GENE <- setdiff(node_GENE_G,node_GENE_K)
  
  print(paste("CN GENE:",length(CN_GENE),sep=" "))
  print(paste("GN GENE:",length(GN_GENE),sep=" "))
  print(paste("LN GENE:",length(LN_GENE),sep=" "))
  
  TFnet$TF_NODE_ATTRIB <- as.factor(ifelse(TFnet$TF %in% CN_TF,"CN",ifelse(TFnet$TF %in% GN_TF,"GN",ifelse(TFnet$TF %in% LN_TF,"LN",""))))
  TFnet$GENE_NODE_ATTRIB <- as.factor(ifelse(TFnet$GENE %in% CN_GENE,"CN",ifelse(TFnet$GENE %in% GN_GENE,"GN",ifelse(TFnet$GENE %in% LN_GENE,"LN",""))))
  
  # edge #
  
  TFnet$TF_GENE <- paste(TFnet$TF,TFnet$GENE,sep="_")
  
  edge_G <- unique(subset(TFnet, SAMPLE=="NORMAL")$TF_GENE)
  edge_K <- unique(subset(TFnet, SAMPLE=="CANCER")$TF_GENE)
  
  common_edge <- intersect(edge_G,edge_K)
  gain_edge <- setdiff(edge_K,edge_G)
  loss_edge <- setdiff(edge_G,edge_K)
  
  TFnet$EDGE_TYPE <- ifelse(TFnet$TF_GENE %in% common_edge,"common",ifelse(TFnet$TF_GENE %in% gain_edge,"gain",ifelse(TFnet$TF_GENE %in% loss_edge,"loss","")))
  TFnet$NODE_TYPE <- ifelse(((TFnet$TF %in% CN_TF) & (TFnet$GENE %in% CN_GENE)),"bothCN","new")
  
  # subset
  TFnet <- subset(TFnet,!(SAMPLE=="NORMAL" & EDGE_TYPE=="common"))
  TFnet <- subset(TFnet, select = -c(SAMPLE))
  
  # attrib 1: common or rewired
  TFnet$EDGE_ATTRIB_1 <- as.factor(ifelse(TFnet$EDGE_TYPE=="common","commonEdge","rewiredEdge"))
  
  # attrib 2: rewired btw CN | GN or LN #
  TFnet$EDGE_ATTRIB_2 <- as.factor(ifelse(TFnet$EDGE_TYPE=="common","commonEdge",
                                          ifelse(TFnet$NODE_TYPE=="bothCN","rewiredCommon",
                                                 ifelse(TFnet$NODE_TYPE=="new","rewiredNew",""))))
  
  # attrib 3: rewiredGain btw CN | rewiredLoss btw CN | rewiredGain btw GN or LN | rewiredLoss btw GN or LN #
  TFnet$EDGE_ATTRIB_3 <- as.factor(ifelse(TFnet$EDGE_TYPE=="common","commonEdge",
                                          ifelse(TFnet$NODE_TYPE=="bothCN" & TFnet$EDGE_TYPE=="gain","rewiredCommonGain",
                                                 ifelse(TFnet$NODE_TYPE=="bothCN" & TFnet$EDGE_TYPE=="loss","rewiredCommonLoss",
                                                        ifelse(TFnet$NODE_TYPE=="new" & TFnet$EDGE_TYPE=="gain","rewiredNewGain",
                                                               ifelse(TFnet$NODE_TYPE=="new" & TFnet$EDGE_TYPE=="loss","rewiredNewLoss",""))))))
  
  return(TFnet)
}
rScore <- function(largeNet, node, nTarget){
  
  # for all edge point "in and out" of a node
  smallNet <- subset(largeNet, TF==node|GENE==node)
  smallNet_out <- subset(smallNet, TF==node)
  smallNet_in <- subset(smallNet, GENE==node)
  
  # fully connected network
  node_TF <- unique(largeNet$TF)
  node_GENE <- unique(largeNet$GENE)
  fully_connected <- 1*nTarget + length(node_TF)*1 - 1
  
  # edge count
  E_outComm <- nrow(smallNet_out[smallNet_out$EDGE_TYPE=="common",])
  E_outGain <- nrow(smallNet_out[smallNet_out$EDGE_TYPE=="gain",])
  E_outLoss <- nrow(smallNet_out[smallNet_out$EDGE_TYPE=="loss",])
  E_inComm <- nrow(smallNet_in[smallNet_in$EDGE_TYPE=="common",])
  E_inGain <- nrow(smallNet_in[smallNet_in$EDGE_TYPE=="gain",])
  E_inLoss <- nrow(smallNet_in[smallNet_in$EDGE_TYPE=="loss",])
  
  # rewiring gain/loss
  R_gainOverLoss <- log10(( E_outGain + E_inGain + 1 ) / ( E_outLoss + E_inLoss + 1 ))
  R_out_gainOverLoss <- log10(( E_outGain + 1 ) / ( E_outLoss + 1 ))
  R_in_gainOverLoss <- log10(( E_inGain + 1 ) / ( E_inLoss + 1 ))
  
  # rewiring outbound/inbound
  R_outOverIn <- log10((E_outGain + E_outLoss + 1 ) / (E_inGain + E_inLoss + 1 ))
  R_gain_outOverIn <- log10(( E_outGain + 1 ) / ( E_inGain + 1 ))
  R_loss_outOverIn <- log10(( E_outLoss + 1 ) / ( E_inLoss + 1 ))
  
  # TF to GENE rewiring
  rScore_TF2GENE <- ( E_outGain + E_inGain + E_outLoss + E_inLoss ) / fully_connected
  
  # TF to TF netowrk
  tf2tfNet <-  subset(largeNet, TF %in% node_TF & GENE %in% node_TF)
  tf2tfNet_out <- subset(tf2tfNet, TF==node)
  tf2tfNet_in <- subset(tf2tfNet, GENE==node)
  
  # fully connected network
  fully_connected.tf2tf <- 1*length(node_TF) + length(node_TF)*1 - 1
  
  # edge count
  E_outComm.tf2tf <- nrow(tf2tfNet_out[tf2tfNet_out$EDGE_TYPE=="common",])
  E_outGain.tf2tf <- nrow(tf2tfNet_out[tf2tfNet_out$EDGE_TYPE=="gain",])
  E_outLoss.tf2tf <- nrow(tf2tfNet_out[tf2tfNet_out$EDGE_TYPE=="loss",])
  E_inComm.tf2tf <- nrow(tf2tfNet_in[tf2tfNet_in$EDGE_TYPE=="common",])
  E_inGain.tf2tf <- nrow(tf2tfNet_in[tf2tfNet_in$EDGE_TYPE=="gain",])
  E_inLoss.tf2tf <- nrow(tf2tfNet_in[tf2tfNet_in$EDGE_TYPE=="loss",])
  
  # TF to TF rewiring
  rScore_TF2TF <- ( E_outGain.tf2tf + E_inGain.tf2tf + E_outLoss.tf2tf + E_inLoss.tf2tf ) / fully_connected.tf2tf
  
  return(c(E_outComm,E_outGain,E_outLoss,E_inComm,E_inGain,E_inLoss,R_gainOverLoss,R_out_gainOverLoss,R_in_gainOverLoss,R_outOverIn,R_gain_outOverIn,R_loss_outOverIn,rScore_TF2GENE,E_outComm.tf2tf,E_outGain.tf2tf,E_outLoss.tf2tf,E_inComm.tf2tf,E_inGain.tf2tf,E_inLoss.tf2tf,rScore_TF2TF))
}
calc_rScore <- function(largeNet, nTarget){
  
  output <- data.frame(E_outComm=numeric(),E_outGain=numeric(),E_outLoss=numeric(),
                       E_inComm=numeric(),E_inGain=numeric(),E_inLoss=numeric(),
                       R_gainOverLoss=numeric(),R_out_gainOverLoss=numeric(),R_in_gainOverLoss=numeric(),
                       R_outOverIn=numeric(),R_gain_outOverIn=numeric(),R_loss_outOverIn=numeric(),
                       rScore_TF2GENE=numeric(),
                       E_outComm.tf2tf=numeric(),E_outGain.tf2tf=numeric(),E_outLoss.tf2tf=numeric(),
                       E_inComm.tf2tf=numeric(),E_inGain.tf2tf=numeric(),E_inLoss.tf2tf=numeric(),
                       rScore_TF2TF=numeric())
  
  tf_list <- unique(sort(as.matrix(largeNet$TF)))
  
  for (tf in tf_list){
    output <- rbind(output,rScore(largeNet,tf,nTarget))
  }
  
  names(output) <- c("E_outComm","E_outGain","E_outLoss",
                     "E_inComm","E_inGain","E_inLoss",
                     "R_gainOverLoss","R_out_gainOverLoss",
                     "R_in_gainOverLoss","R_outOverIn",
                     "R_gain_outOverIn","R_loss_outOverIn",
                     "rScore_TF2GENE",
                     "E_outComm.tf2tf","E_outGain.tf2tf","E_outLoss.tf2tf",
                     "E_inComm.tf2tf","E_inGain.tf2tf","E_inLoss.tf2tf",
                     "rScore_TF2TF")
  output$TF <- tf_list
  row.names(output) <- tf_list
  
  return(output)
}

# load data
path_tip_nor <- "example/edgeList_TIP_GM12878.tsv"
path_tip_can <- "example/edgeList_TIP_K562.tsv"
path_enh_nor <- "example/edgeList_ENH_GM12878.tsv"
path_enh_can <- "example/edgeList_ENH_K562.tsv"
path_tss_nor <- "example/edgeList_uniqTSS2500_GM12878.tsv"
path_tss_can <- "example/edgeList_uniqTSS2500_K562.tsv"
path_tss_esc <- "example/edgeList_uniqTSS2500_H1-hESC.tsv"
path_biotype <- "example/gencode.v19.gene.biotype.txt"
data.tf.type <- read.csv("example/commonTF_annotation.csv",header=T,sep=",")
tf.com <- data.tf.type[data.tf.type$MAJOR_CLASS!="general",]$TF
master_gene_feature <- read.csv("example/master_gene_feature_sigAvg200_uniqTSS_log2.csv",header = T)

# save common TF list
# write.table(tf.com,"KG_TF.lst",row.names = F,col.names = F,quote = F)

#####################################
### TIP+Enhancer network analysis ###
#####################################

network_name <- "TIP_ENH"
mnet.tip_enh <- networkAnalysis(loadTIPNet(path_tip_nor,path_tip_can,path_enh_nor,path_enh_can),tf.com,path_biotype)
write.csv(mnet.tip_enh,paste(network_name,"mnet.csv",sep="_"),row.names=F,quote=F)

tfnet.tip_enh <- subset(mnet.tip_enh, GENE %in% tf.com)
tfnet.tip_enh$TF <- factor(tfnet.tip_enh$TF)
write.csv(tfnet.tip_enh,paste(network_name,"tfnet.csv",sep="_"),row.names=F,quote=F)

# TF attrib
mnet_rScore.tip_enh <- calc_rScore(mnet.tip_enh, nrow(master_gene_feature))
write.csv(mnet_rScore.tip_enh,paste(network_name,"TFattrib.csv",sep="_"),row.names=F,quote=F)

mnet.tip <- subset(mnet.tip_enh,SRC=="TIP")
mnet.enh <- subset(mnet.tip_enh,SRC=="ENH")
mnet_rScore.tip <- calc_rScore(mnet.tip, nrow(master_gene_feature))
mnet_rScore.enh <- calc_rScore(mnet.enh, nrow(master_gene_feature))

mnet_rScore.tip_enh$rScore_TF2GENE_new <- sign(mnet_rScore.tip_enh$R_gainOverLoss)*mnet_rScore.tip_enh$rScore_TF2GENE
mnet_rScore.tip$rScore_TF2GENE_new <- sign(mnet_rScore.tip$R_gainOverLoss)*mnet_rScore.tip$rScore_TF2GENE
mnet_rScore.enh$rScore_TF2GENE_new <- sign(mnet_rScore.enh$R_gainOverLoss)*mnet_rScore.enh$rScore_TF2GENE

# GENE attrib
targetGENE <- unique(union(mnet.tip$TF,mnet.tip$GENE))
gene_attrib <- subset(master_gene_feature, GENE %in% targetGENE)

# save
# write.csv(gene_attrib,paste(network_name,"GENEattrib.csv",sep="_"),row.names=F,quote=F)

### rScore (rewiring score) barplot
barplot_TIP_ENH <- function(){
  
  dat.tip_enh <- subset(mnet_rScore.tip_enh, select = c(TF, rScore_TF2GENE_new))
  names(dat.tip_enh) <- c("TF","rScore_TIP+ENH")
  dat.tip <- subset(mnet_rScore.tip, select = c(TF, rScore_TF2GENE_new))
  names(dat.tip) <- c("TF","rScore_TIP")
  dat.enh <- subset(mnet_rScore.enh, select = c(TF, rScore_TF2GENE_new))
  names(dat.enh) <- c("TF","rScore_ENH")
  
  dat_merged <- merge(dat.tip_enh,dat.tip,all.x=T,by="TF")
  dat_merged <- merge(dat_merged,dat.enh,all.x=T,by="TF")
  
  dat_melt <- melt(dat_merged, id.vars=1)
  dat_melt$TF <- factor(dat_melt$TF, levels = dat_melt$TF[order(dat_melt[dat_melt$variable=="rScore_TIP+ENH",]$value,decreasing = F)])
  
  ### normalize
  dat_melt$value <- dat_melt$value/max(dat_melt$value)
  
  ### mixedM
  mixedM <- read.table("example/TFrand_mixedM_median_cubicroot.txt",header=T)
  names(mixedM) <- c("mixedM_TSS+ENH","mixedM_TSS","mixedM_ENH")
  mixedM <- subset(mixedM,select=c("mixedM_TSS+ENH"))
  mixedM <- mixedM/max(abs(mixedM))
  mixedM$TF <- row.names(mixedM)
  
  mixedM_melt <- melt(mixedM, id.vars="TF")
  
  # merge
  dat_melt <- rbind(dat_melt,mixedM_melt)
  
  # TIP + ENH + MixedM + cancer-related
  type_melt <- melt(data.tf.type, id.vars="TF")
  type_melt$value[is.na(type_melt$value)] <- ""
  type_melt <- type_melt[type_melt$variable=="cancer.related",]
  type_melt <- type_melt[type_melt$TF %in% dat_merged$TF,]
  
  ggplot() + 
    geom_point(data=type_melt[type_melt$variable=="cancer.related",], aes(x=reorder(TF,dat_melt[dat_melt$variable=="rScore_TIP+ENH",]$value), y=-1, color=value), stat="identity", size=1) +
    geom_bar(data=dat_melt[dat_melt$variable=="rScore_TIP+ENH"|dat_melt$variable=="rScore_TIP"|dat_melt$variable=="rScore_ENH",], aes(x=reorder(TF,value), y=abs(value), fill=variable), color="grey50", stat="identity", position="dodge", width=.6) + 
    geom_bar(data=dat_melt[dat_melt$variable=="mixedM_TSS+ENH",], aes(x=TF, y=-abs(value), fill=variable), color="grey50", stat="identity", position="dodge", width=.2) + 
    
    geom_line(data=dat_melt[dat_melt$variable=="rScore_TIP+ENH",], aes(x=TF, y=abs(value), group=variable), col="darkgrey") +
    scale_y_continuous("Normalized Rewiring Score", limits = c(-1,1), breaks = c(-1,0,1), labels = c(1,0,1)) +
    scale_fill_brewer(limits = c("rScore_TIP","rScore_ENH","rScore_TIP+ENH","mixedM_TSS+ENH"),
                      labels = c("Proximal","Distal","Prox + Dist","Gene Topic"),
                      palette = "Greys", direction = -1) +
    theme(text = element_text(face = "bold", size = 12),
          axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, face = "bold", size = 14), axis.text.y = element_text(face = "bold", size = 12),
          axis.title.x = element_blank(), axis.title.y = element_text(face = "bold", size = 14),
          legend.title = element_blank(), legend.position = c(0.5, 0.8), legend.direction = "vertical", legend.text = element_text(face = "bold", size = 12),
          panel.background = element_blank())
  
  # save
  write.csv(dat_melt,paste(network_name,"rewiringIndex_TIP+ENH.csv",sep="_"),row.names=F,quote=F)
}

#####################################
### TSS+Enhancer network analysis ###
#####################################

network_name <- "TSS_ENH"
mnet.tss_enh <- networkAnalysis(loadTSSNet(path_tss_nor,path_tss_can,path_enh_nor,path_enh_can),tf.com,path_biotype)
write.csv(mnet.tss_enh,paste(network_name,"mnet.csv",sep="_"),row.names=F,quote=F)

tfnet.tss_enh <- subset(mnet.tss_enh, GENE %in% tf.com)
tfnet.tss_enh$TF <- factor(tfnet.tss_enh$TF)
write.csv(tfnet.tss_enh,paste(network_name,"tfnet.csv",sep="_"),row.names=F,quote=F)

# TF attrib
mnet_rScore.tss_enh <- calc_rScore(mnet.tss_enh, nrow(master_gene_feature))
write.csv(mnet_rScore.tss_enh,paste(network_name,"TFattrib.csv",sep="_"),row.names=F,quote=F)

mnet.tss <- subset(mnet.tss_enh,SRC=="TSS")
mnet.enh <- subset(mnet.tss_enh,SRC=="ENH")
mnet_rScore.tss <- calc_rScore(mnet.tss, nrow(master_gene_feature))
mnet_rScore.enh <- calc_rScore(mnet.enh, nrow(master_gene_feature))

mnet_rScore.tss_enh$rScore_TF2GENE_new <- sign(mnet_rScore.tss_enh$R_gainOverLoss)*mnet_rScore.tss_enh$rScore_TF2GENE
mnet_rScore.tss$rScore_TF2GENE_new <- sign(mnet_rScore.tss$R_gainOverLoss)*mnet_rScore.tss$rScore_TF2GENE
mnet_rScore.enh$rScore_TF2GENE_new <- sign(mnet_rScore.enh$R_gainOverLoss)*mnet_rScore.enh$rScore_TF2GENE

# GENE attrib
targetGENE <- unique(union(mnet.tss_enh$TF,mnet.tss$GENE))
gene_attrib <- subset(master_gene_feature, GENE %in% targetGENE)

# save
# write.csv(gene_attrib,paste(network_name,"GENEattrib.csv",sep="_"),row.names=F,quote=F)

### rScore (rewiring score) barplot
barplot_TSS_ENH <- function(){
  
  dat.tss_enh <- subset(mnet_rScore.tss_enh, select = c(TF, rScore_TF2GENE_new))
  names(dat.tss_enh) <- c("TF","rScore_TSS+ENH")
  dat.tss <- subset(mnet_rScore.tss, select = c(TF, rScore_TF2GENE_new))
  names(dat.tss) <- c("TF","rScore_TSS")
  dat.enh <- subset(mnet_rScore.enh, select = c(TF, rScore_TF2GENE_new))
  names(dat.enh) <- c("TF","rScore_ENH")
  
  dat_merged <- merge(dat.tss_enh,dat.tss,all.x=T,by="TF")
  dat_merged <- merge(dat_merged,dat.enh,all.x=T,by="TF")
  
  dat_melt <- melt(dat_merged, id.vars=1)
  dat_melt$TF <- factor(dat_melt$TF, levels = dat_melt$TF[order(dat_melt[dat_melt$variable=="rScore_TSS+ENH",]$value,decreasing = F)])
  
  # normalize
  dat_melt$value <- dat_melt$value/max(dat_melt$value)
  
  # mixedM
  mixedM <- read.table("example/TFrand_mixedM_median_cubicroot.txt",header=T)
  mixedM <- mixedM/max(mixedM)
  names(mixedM) <- c("mixedM_TSS+ENH","mixedM_TSS","mixedM_ENH")
  mixedM$TF <- row.names(mixedM)
  mixedM <- subset(mixedM,select=c("TF","mixedM_TSS+ENH"))
  mixedM_melt <- melt(mixedM, id.vars="TF")
  
  # merge
  dat_melt <- rbind(dat_melt,mixedM_melt)
  
  ggplot(dat_melt) +
    geom_bar(aes(x=TF, y=value, fill=variable), stat="identity", position="dodge", col="black") +
    theme(text = element_text(face = "bold", size = 16),
          axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, face = "bold", size = 16), axis.text.y = element_text(face = "bold", size = 12),
          axis.title.x = element_blank(), axis.title.y = element_text(face = "bold", size = 14),
          legend.title = element_blank(), legend.position = c(0.1, 0.85), legend.direction = "vertical", legend.text = element_text(face = "bold", size = 14),
          panel.background = element_blank()) +
    scale_y_continuous("Normalized Rewiring Score")+
    scale_fill_brewer(limits = c("rScore_TSS+ENH","rScore_TSS","rScore_ENH","mixedM_TSS+ENH"),
                      labels = c("Proximal + Distal","Proximal [TSS]","Distal [Enhancer]","mixedM_TSS+ENH"),
                      palette = "Set1", direction = 1)
  
  # save
  write.csv(dat_melt,paste(network_name,"rewiringIndex_TSS+ENH.csv",sep="_"),row.names=F,quote=F)
}

##########################################
### Tumor-normal expression comparison ###
##########################################

rewired_tpm <- merge(dat_merged,master_gene_feature,by.x="TF",by.y="GENE")
rewired_tpm <- subset(rewired_tpm,select=c("TF","rScore_TIP+ENH","GENE_TPM_log2fc"))
ggplot(rewired_tpm)+
  geom_bar(aes(x=reorder(TF,`rScore_TIP+ENH`),y=GENE_TPM_log2fc), stat = "identity", width=.8)+
  scale_y_continuous("log2 fold-change [K/G]", limits = c(-10,13)) +
  theme(text = element_text(face = "bold", size = 12),
        axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, face = "bold", size = 14), axis.text.y = element_text(face = "bold", size = 12),
        axis.title.x = element_blank(), axis.title.y = element_text(face = "bold", size = 14),
        panel.background = element_blank())

#######################################################
### Direction of rewiring w.r.t stem cell (H1-hESC) ###
#######################################################

directionRewiringH1 <- function(path_tss_nor,path_tss_can,path_tss_esc,path_biotype){
  
  # load edge list
  edge_tss_nor <- fread(path_tss_nor,header=F,sep="\t")
  edge_tss_can <- fread(path_tss_can,header=F,sep="\t")
  edge_tss_esc <- fread(path_tss_esc,header=F,sep="\t")
  
  colnames(edge_tss_nor) <- c("SAMPLE","TF","GENE","SRC")
  colnames(edge_tss_can) <- c("SAMPLE","TF","GENE","SRC")
  colnames(edge_tss_esc) <- c("SAMPLE","TF","GENE","SRC")

  merged_net <- rbind(edge_tss_nor,edge_tss_can)
  merged_net <- rbind(merged_net,edge_tss_esc)
  
  # subset commonTF #
  
  uniq_tf_h <- unique(merged_net[merged_net$SAMPLE=="H1-hESC",]$TF)
  uniq_tf_kgh <- intersect(tf.com,uniq_tf_h)
  
  merged_net <- subset(merged_net,TF %in% uniq_tf_kgh)
  
  # subset protein_coding GENE #
  
  dgene.type <- fread(path_biotype,header=T,sep="\t")
  dgene.type <- data.frame(dgene.type)
  dgene.type <- data.frame(lapply(dgene.type, as.character), stringsAsFactors=FALSE)
  gene.coding <- dgene.type$gene_name[dgene.type$gene_type=="protein_coding"]
  
  merged_net <- subset(merged_net, GENE %in% gene.coding)
  write.csv(merged_net,"h1_merged_net.csv",row.names = F)
  
  # tf node #
  
  node_TF_G <- unique(subset(merged_net, SAMPLE=="GM12878")$TF)
  node_TF_K <- unique(subset(merged_net, SAMPLE=="K562")$TF)
  node_TF_H <- unique(subset(merged_net, SAMPLE=="H1-hESC")$TF)
  
  # gene node #
  
  node_GENE_G <- unique(subset(merged_net, SAMPLE=="GM12878")$GENE)
  node_GENE_K <- unique(subset(merged_net, SAMPLE=="K562")$GENE)
  node_GENE_H <- unique(subset(merged_net, SAMPLE=="H1-hESC")$GENE)
  
  ###
  
  out <- data.frame(TF=uniq_tf_kgh,K=0,G=0,H=0,K_only=0,G_only=0,H_only=0,KxG_only=0,KxH_only=0,GxH_only=0,KxGxH=0,same.dir=0,opposite.dir=0,overall.dir=0)
  
  for(tf in uniq_tf_kgh){
    tfsubset <- subset(merged_net, TF==tf)
    
    target_g <- subset(tfsubset, SAMPLE=="GM12878")$GENE
    target_k <- subset(tfsubset, SAMPLE=="K562")$GENE
    target_h <- subset(tfsubset, SAMPLE=="H1-hESC")$GENE
    
    out[out$TF==tf,]$K <- length( target_k )
    out[out$TF==tf,]$G <- length( target_g )
    out[out$TF==tf,]$H <- length( target_h )
    
    out[out$TF==tf,]$KxGxH <- length( intersect(target_k, intersect(target_g,target_h)) )
    
    out[out$TF==tf,]$KxG_only <- length( setdiff(intersect(target_k,target_g), target_h) )
    out[out$TF==tf,]$KxH_only <- length( setdiff(intersect(target_k,target_h), target_g) )
    out[out$TF==tf,]$GxH_only <- length( setdiff(intersect(target_g,target_h), target_k) )
    
    out[out$TF==tf,]$K_only <- length( setdiff(target_k, union(target_g,target_h)) )
    out[out$TF==tf,]$G_only <- length( setdiff(target_g, union(target_k,target_h)) )
    out[out$TF==tf,]$H_only <- length( setdiff(target_h, union(target_k,target_g)) )
    
    out[out$TF==tf,]$same.dir <- length( union( setdiff(target_g, union(target_k,target_h)), setdiff(intersect(target_k,target_h),target_g)) )
    out[out$TF==tf,]$opposite.dir <- length( union( setdiff(target_k, union(target_g,target_h)), setdiff(intersect(target_g,target_h), target_k) ) )
    
    out[out$TF==tf,]$overall.dir <- out[out$TF==tf,]$same.dir - out[out$TF==tf,]$opposite.dir
    
  }
  
  write.csv(out,"h1_similarity.csv",row.names = F)
  
}
